@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -84,14 +82,14 @@ void
 addarc(
 nltype *parentp,
 nltype *childp,
-unsigned long count,
-unsigned long order)
+uint32_t count,
+uint32_t order)
 {
     arctype *arcp;
 
 #ifdef DEBUG
 	if(debug & TALLYDEBUG){
-	    printf("[addarc] %lu arcs from %s to %s\n" ,
+	    printf("[addarc] %u arcs from %s to %s\n" ,
 		   count, parentp->name, childp->name);
 	}
 #endif
@@ -102,7 +100,7 @@ unsigned long order)
 	     */
 #ifdef DEBUG
 	    if(debug & TALLYDEBUG){
-		printf("[tally] hit %ld += %lu\n",
+		printf("[tally] hit %u += %u\n",
 		       arcp->arc_count, count);
 	    }
 #endif
@@ -151,7 +149,7 @@ void)
 {
     nltype	*parentp, **timesortnlp;
     arctype	*arcp;
-    long	index;
+    uint32_t index;
 
 	/*
 	 * initialize various things:
@@ -244,7 +242,7 @@ void)
 	for(index = 0; index < nname; index++){
 	    timesortnlp[index] = &nl[index];
 	}
-	for(index = 1; index <= ncycle; index++){
+	for(index = 1; index <= (uint32_t)ncycle; index++){
 	    timesortnlp[nname + index - 1] = &cyclenl[index];
 	}
 	qsort(timesortnlp, nname + ncycle, sizeof(nltype *),
@@ -260,7 +258,7 @@ void
 dotime(
 void)
 {
-    int	index;
+    uint32_t index;
 
 	cycletime();
 	for(index = 0; index < nname; index += 1){
@@ -345,7 +343,7 @@ nltype *parentp)
 	    if(debug & PROPDEBUG){
 		printf("[dotime] child \t");
 		printname(childp);
-		printf(" with %f %f %ld/%ld\n",
+		printf(" with %f %f %u/%u\n",
 			childp->time, childp->childtime,
 			arcp->arc_count, childp->ncall);
 		printf("[dotime] parent\t");
@@ -486,7 +484,7 @@ void
 doflags(
 void)
 {
-    int index;
+    int32_t index;
     nltype *childp;
     nltype *oldhead;
 
