@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -54,6 +52,8 @@
 #ifndef	_EXEC_
 #define	_EXEC_	1
 
+#include <stdint.h>
+
 #ifndef NeXT_MOD
 #define NeXT_MOD 1
 #endif
@@ -65,16 +65,16 @@ struct exec {
 #if	sun || NeXT_MOD
 unsigned short  a_machtype;     /* machine type */
 unsigned short  a_magic;        /* magic number */
-#else	sun || NeXT_MOD
-	long	a_magic;	/* magic number */
-#endif	sun || NeXT_MOD
-unsigned long	a_text;		/* size of text segment */
-unsigned long	a_data;		/* size of initialized data */
-unsigned long	a_bss;		/* size of uninitialized data */
-unsigned long	a_syms;		/* size of symbol table */
-unsigned long	a_entry;	/* entry point */
-unsigned long	a_trsize;	/* size of text relocation */
-unsigned long	a_drsize;	/* size of data relocation */
+#else	/* sun || NeXT_MOD */
+	int32_t	a_magic;	/* magic number */
+#endif	/* sun || NeXT_MOD */
+uint32_t	a_text;		/* size of text segment */
+uint32_t	a_data;		/* size of initialized data */
+uint32_t	a_bss;		/* size of uninitialized data */
+uint32_t	a_syms;		/* size of symbol table */
+uint32_t	a_entry;	/* entry point */
+uint32_t	a_trsize;	/* size of text relocation */
+uint32_t	a_drsize;	/* size of data relocation */
 };
 
 #define	OMAGIC	0407		/* old impure format */
@@ -87,6 +87,6 @@ unsigned long	a_drsize;	/* size of data relocation */
 #define M_OLDSUN2	0	/* old sun-2 executable files */
 #define M_68010		1	/* runs on either 68010 or 68020 */
 #define M_68020		2	/* runs only on 68020 */
-#endif sun
+#endif /* sun */
 
-#endif	_EXEC_
+#endif	/* _EXEC_ */
