@@ -122,7 +122,7 @@ __private_extern__ struct hints_info output_hints_info = { { 0 } };
 __private_extern__ struct cksum_info output_cksum_info = { { 0 } };
 
 /*
- * The output file's prebind_cksum load command.
+ * The output file's UUID load command.
  */
 __private_extern__ struct uuid_info output_uuid_info = { 0 };
 
@@ -156,6 +156,8 @@ static struct hp_pa_frame_thread_state hppa_frame_state = { 0 };
 static struct hp_pa_integer_thread_state hppa_integer_state = { 0 };
 /* cputype == CPU_TYPE_SPARC, all subtypes */
 static struct sparc_thread_state_regs sparc_state = { {0} };
+/* cputype == CPU_TYPE_ARM, all subtypes */
+static arm_thread_state_t arm_state = { {0} };
 
 static void layout_segments(void);
 static unsigned long next_vmaddr(
@@ -191,7 +193,9 @@ layout(void)
 	memset(&output_dysymtab_info, '\0', sizeof(struct dysymtab_info));
 	memset(&output_hints_info, '\0', sizeof(struct hints_info));
 	memset(&output_cksum_info, '\0', sizeof(struct cksum_info));
+#ifndef KLD
 	memset(&output_uuid_info, '\0', sizeof(struct uuid_info));
+#endif
 	memset(&output_thread_info, '\0', sizeof(struct thread_info));
 	memset(&mc680x0, '\0', sizeof(struct m68k_thread_state_regs));
 	memset(&powerpc,     '\0', sizeof(ppc_thread_state_t));
@@ -425,7 +429,7 @@ layout_rld_symfile(void)
 						sizeof(struct symtab_command);
 	output_symtab_info.symtab_command.nsyms = nmerged_symbols;
 	output_symtab_info.symtab_command.strsize =
-	    round(merged_string_size + STRING_SIZE_OFFSET,
+	    rnd(merged_string_size + STRING_SIZE_OFFSET,
 		  sizeof(unsigned long));
 	output_symtab_info.output_strpad =
 	    output_symtab_info.symtab_command.strsize -
@@ -742,7 +746,7 @@ layout_segments(void)
 			     + nlocal_symbols
 			     - nmerged_symbols_referenced_only_from_dylibs) *
 			    sizeof(struct nlist) +
-			    round(merged_string_size +
+			    rnd(merged_string_size +
 				  local_string_size +
 				  STRING_SIZE_OFFSET,
 				  sizeof(unsigned long));
@@ -756,7 +760,7 @@ layout_segments(void)
 			    output_hints_info.twolevel_hints_command.nhints *
 			    sizeof(struct twolevel_hint);
 		    linkedit_segment.sg.vmsize =
-				round(linkedit_segment.sg.filesize, segalign);
+				rnd(linkedit_segment.sg.filesize, segalign);
 		    /* place this last in the merged segment list */
 		    p = &merged_segments;
 		    while(*p){
@@ -809,7 +813,7 @@ layout_segments(void)
 		    pagezero_segment.prot_set = TRUE;
 		    strcpy(pagezero_segment.sg.segname, SEG_PAGEZERO);
 		    if(pagezero_size != 0)
-			pagezero_segment.sg.vmsize = round(pagezero_size,
+			pagezero_segment.sg.vmsize = rnd(pagezero_size,
 							   segalign);
 		    else
 			pagezero_segment.sg.vmsize = segalign;
@@ -916,7 +920,8 @@ layout_segments(void)
 			  (unsigned int)segalign);
 	    }
 	    else{
-		segs_read_write_addr = segs_read_only_addr + 0x10000000;
+		segs_read_write_addr = segs_read_only_addr + 
+				   get_shared_region_size_from_flag(&arch_flag);
 	    }
 	}
 	first_msg = merged_segments;
@@ -1122,7 +1127,7 @@ layout_segments(void)
 		+ nlocal_symbols
 		- nmerged_symbols_referenced_only_from_dylibs;
 	    output_symtab_info.symtab_command.strsize =
-		round(merged_string_size +
+		rnd(merged_string_size +
 		      local_string_size +
 		      STRING_SIZE_OFFSET,
 		      sizeof(unsigned long));
@@ -1160,8 +1165,7 @@ layout_segments(void)
 	/*
 	 * Create the prebind cksum load command.
 	 */
-	if(prebinding == TRUE &&
-	   macosx_deployment_target >= MACOSX_DEPLOYMENT_TARGET_10_2){
+	if(prebinding == TRUE && macosx_deployment_target.major >= 2){
 	    output_cksum_info.prebind_cksum_command.cmd = LC_PREBIND_CKSUM;
 	    output_cksum_info.prebind_cksum_command.cmdsize =
 					sizeof(struct prebind_cksum_command);
@@ -1172,13 +1176,22 @@ layout_segments(void)
 	/*
 	 * Create the uuid load command.
 	 */
-	if(output_uuid_info.suppress != TRUE && output_uuid_info.emit == TRUE){
+#ifndef KLD
+	if(output_uuid_info.suppress != TRUE &&
+	   (output_uuid_info.emit == TRUE ||
+	    arch_flag.cputype == CPU_TYPE_ARM)){
 	    output_uuid_info.uuid_command.cmd = LC_UUID;
 	    output_uuid_info.uuid_command.cmdsize = sizeof(struct uuid_command);
 	    uuid(&(output_uuid_info.uuid_command.uuid[0]));
 	    ncmds++;
 	    sizeofcmds += output_uuid_info.uuid_command.cmdsize;
 	}
+#else
+	if(output_uuid_info.uuid_command.cmdsize != 0){
+	    ncmds++;
+	    sizeofcmds += output_uuid_info.uuid_command.cmdsize;
+	}
+#endif /* KLD */
 
 	/*
 	 * Create the thread command if this is filetype is to have one.
@@ -1267,6 +1280,15 @@ layout_segments(void)
 	      output_thread_info.thread_command.cmdsize += sizeof(long) *
 		SPARC_THREAD_STATE_REGS_COUNT;
 	    }
+	    else if (arch_flag.cputype == CPU_TYPE_ARM) {
+	      output_thread_info.flavor = ARM_THREAD_STATE;
+	      output_thread_info.count = ARM_THREAD_STATE_COUNT;
+	      output_thread_info.entry_point = (int *)&(arm_state.__pc);
+	      output_thread_info.stack_pointer = (int *)&(arm_state.__sp);
+	      output_thread_info.state = &arm_state;
+	      output_thread_info.thread_command.cmdsize += sizeof(long) *
+		ARM_THREAD_STATE_COUNT;
+	    }
 	    else{
 		fatal("internal error: layout_segments() called with unknown "
 		      "cputype (%d) set", arch_flag.cputype);
@@ -1331,7 +1353,7 @@ layout_segments(void)
 	    headers_size = segalign;
 	}
 	else if(filetype == MH_DYLINKER){
-	    headers_size = round(headers_size, segalign);
+	    headers_size = rnd(headers_size, segalign);
 	}
 
 	/*
@@ -1356,16 +1378,16 @@ layout_segments(void)
 				  (unsigned int)(1 << ms->s.align),
 				  ms->s.segname, ms->s.sectname,
 				  (unsigned int)segalign);
-			size = round(size, 1 << ms->s.align);
+			size = rnd(size, 1 << ms->s.align);
 			if((unsigned long)(1 << ms->s.align) > max_first_align)
 			    max_first_align = 1 << ms->s.align;
 			size += ms->s.size;
 			content = &(ms->next);
 		    }
 		    if(errors == 0){
-			pad = ((round(size + round(headers_size,
+			pad = ((rnd(size + rnd(headers_size,
 				      max_first_align), segalign) -
-			       (size + round(headers_size, max_first_align))) /
+			       (size + rnd(headers_size, max_first_align))) /
 				 max_first_align) * max_first_align;
 			if(pad > headerpad)
 			    headerpad = pad;
@@ -1405,7 +1427,7 @@ layout_segments(void)
 			      ms->s.sectname, (unsigned int)segalign);
 		    if((unsigned long)(1 << ms->s.align) > max_align)
 			max_align = 1 << ms->s.align;
-		    addr = round(addr, 1 << ms->s.align);
+		    addr = rnd(addr, 1 << ms->s.align);
 		    ms->s.addr = addr;
 		    addr += ms->s.size;
 		    content = &(ms->next);
@@ -1413,7 +1435,7 @@ layout_segments(void)
 		if(msg == &object_segment)
 		    msg->sg.filesize = addr;
 		else
-		    msg->sg.filesize = round(addr, segalign);
+		    msg->sg.filesize = rnd(addr, segalign);
 		zerofill = &(msg->zerofill_sections);
 		while(*zerofill){
 		    ms = *zerofill;
@@ -1424,7 +1446,7 @@ layout_segments(void)
 			      ms->s.sectname, (unsigned int)segalign);
 		    if((unsigned long)(1 << ms->s.align) > max_align)
 			max_align = 1 << ms->s.align;
-		    addr = round(addr, 1 << ms->s.align);
+		    addr = rnd(addr, 1 << ms->s.align);
 		    ms->s.addr = addr;
 		    addr += ms->s.size;
 		    zerofill = &(ms->next);
@@ -1432,7 +1454,7 @@ layout_segments(void)
 		if(msg == &object_segment)
 		    msg->sg.vmsize = addr;
 		else
-		    msg->sg.vmsize = round(addr, segalign);
+		    msg->sg.vmsize = rnd(addr, segalign);
 	    }
 	    p = &(msg->next);
 	}
@@ -1446,7 +1468,7 @@ layout_segments(void)
 	 */
 	output_size = 0;
 
-	headers_size = round(headers_size, max_align);
+	headers_size = rnd(headers_size, max_align);
 	output_size = headers_size;
 	if(first_msg != NULL)
 	    output_size += first_msg->sg.vmsize;
@@ -1644,7 +1666,7 @@ layout_segments(void)
 	 * sizeof(long) so the offset must be rounded to this as the sections
 	 * and segments may not be rounded to this.
 	 */
-	offset = round(offset, sizeof(long));
+	offset = rnd(offset, sizeof(long));
 #ifdef RLD
 	/*
 	 * For RLD if there is any symbol table it is written past the size
@@ -1848,8 +1870,14 @@ layout_segments(void)
 		   merged_symbol->nlist.n_type == (N_EXT | N_UNDF))
 		    fatal("initialization routine symbol name: %s not defined",
 			  init_name);
-		output_routines_info.routines_command.init_address =
-		    merged_symbol->nlist.n_value;
+		if(arch_flag.cputype == CPU_TYPE_ARM &&
+		   (merged_symbol->nlist.n_desc & N_ARM_THUMB_DEF))
+		    /* Have to set the low-order bit if symbol is Thumb */
+		    output_routines_info.routines_command.init_address =
+			merged_symbol->nlist.n_value | 1;
+		else
+		    output_routines_info.routines_command.init_address =
+			merged_symbol->nlist.n_value;
 		output_routines_info.routines_command.init_module =
 		    merged_symbol->definition_object->imodtab;
 	    }
