@@ -94,13 +94,16 @@
 #include "stuff/errors.h"
 #include "stuff/allocate.h"
 #include "stuff/guess_short_name.h"
+#ifdef LTO_SUPPORT
+#include "stuff/lto.h"
+#endif /* LTO_SUPPORT */
 
 /* used by error routines as the name of the program */
 char *progname = NULL;
 
 /* flags set from the command line arguments */
 struct cmd_flags {
-    unsigned long nfiles;
+    uint32_t nfiles;
     enum bool a;	/* print all symbol table entries including stabs */
     enum bool g;	/* print only global symbols */
     enum bool n;	/* sort numericly rather than alphabetically */
@@ -108,6 +111,7 @@ struct cmd_flags {
     enum bool p;	/* don't sort; print in symbol table order */
     enum bool r;	/* sort in reverse direction */
     enum bool u;	/* print only undefined symbols */
+    enum bool U;	/* only undefined symbols */
     enum bool m;	/* print symbol in Mach-O symbol format */
     enum bool x;	/* print the symbol table entry in hex and the name */
     enum bool j;	/* just print the symbol name (no value or type) */
@@ -120,7 +124,7 @@ struct cmd_flags {
     enum bool b;	/* print only stabs for the following include */
     char *bincl_name;	/*  the begin include name for -b */
     enum bool i;	/* start searching for begin include at -iN index */
-    unsigned long index;/*  the index to start searching at */
+    uint32_t index;	/*  the index to start searching at */
     enum bool A;	/* pathname or library name of an object on each line */
     enum bool P;	/* portable output format */
     char *format;	/* the -t format */
@@ -128,23 +132,24 @@ struct cmd_flags {
 /* These need to be static because of the qsort compare function */
 static struct cmd_flags cmd_flags = { 0 };
 static char *strings = NULL;
-static unsigned long strsize = 0;
+static uint32_t strsize = 0;
+static enum bool compare_lto = FALSE;
 
 /* flags set by processing a specific object file */
 struct process_flags {
-    unsigned long nsect;	/* The nsect, address and size for the */
+    uint32_t nsect;		/* The nsect, address and size for the */
     uint64_t sect_addr,		/*  section specified by the -s flag */
 	     sect_size;
     enum bool sect_start_symbol;/* For processing the -l flag, set if a */
 				/*  symbol with the start address of the */
 				/*  section is found */
-    unsigned long nsects;	/* For printing the symbol types, the number */
+    uint32_t nsects;		/* For printing the symbol types, the number */
     struct section **sections;	/*  of sections and an array of section ptrs */
     struct section_64 **sections64;
     unsigned char text_nsect,	/* For printing symbols types, T, D, and B */
 		  data_nsect,	/*  for text, data and bss symbols */
 		  bss_nsect;
-    unsigned long nlibs;	/* For printing the twolevel namespace */
+    uint32_t nlibs;		/* For printing the twolevel namespace */
     char **lib_names;		/*  references types, the number of libraries */
 				/*  an array of pointers to library names */
 };
@@ -166,13 +171,23 @@ static void nm(
     struct ofile *ofile,
     char *arch_name,
     void *cookie);
+#ifdef LTO_SUPPORT
+static void nm_lto(
+    struct ofile *ofile,
+    char *arch_name,
+    struct cmd_flags *cmd_flags);
+#endif /* LTO_SUPPORT */
+static void print_header(
+    struct ofile *ofile,
+    char *arch_name,
+    struct cmd_flags *cmd_flags);
 static struct symbol *select_symbols(
     struct ofile *ofile,
     struct symtab_command *st,
     struct dysymtab_command *dyst,
     struct cmd_flags *cmd_flags,
     struct process_flags *process_flags,
-    unsigned long *nsymbols);
+    uint32_t *nsymbols);
 static void make_symbol_32(
     struct symbol *symbol,
     struct nlist *nl);
@@ -186,18 +201,18 @@ static enum bool select_symbol(
 static void print_mach_symbols(
     struct ofile *ofile,
     struct symbol *symbols,
-    unsigned long nsymbols,
+    uint32_t nsymbols,
     char *strings,
-    unsigned long strsize,
+    uint32_t strsize,
     struct cmd_flags *cmd_flags,
     struct process_flags *process_flags,
     char *arch_name);
 static void print_symbols(
     struct ofile *ofile,
     struct symbol *symbols,
-    unsigned long nsymbols,
+    uint32_t nsymbols,
     char *strings,
-    unsigned long strsize,
+    uint32_t strsize,
     struct cmd_flags *cmd_flags,
     struct process_flags *process_flags,
     char *arch_name,
@@ -211,6 +226,10 @@ static int value_diff_compare(
     struct value_diff *p1,
     struct value_diff *p2);
 
+/* apple_version is created by the libstuff/Makefile */
+extern char apple_version[];
+char *version = apple_version;
+
 int
 main(
 int argc,
@@ -218,9 +237,9 @@ char **argv,
 char **envp)
 {
     int i;
-    unsigned long j;
+    uint32_t j;
     struct arch_flag *arch_flags;
-    unsigned long narch_flags;
+    uint32_t narch_flags;
     enum bool all_archs;
     char **files;
 
@@ -238,6 +257,7 @@ char **envp)
 	cmd_flags.p = FALSE;
 	cmd_flags.r = FALSE;
 	cmd_flags.u = FALSE;
+	cmd_flags.U = FALSE;
 	cmd_flags.m = FALSE;
 	cmd_flags.x = FALSE;
 	cmd_flags.j = FALSE;
@@ -332,8 +352,19 @@ char **envp)
 			    cmd_flags.r = TRUE;
 			    break;
 			case 'u':
+			    if(cmd_flags.U == TRUE){
+				error("can't specifiy both -u and -U");
+				usage();
+			    }
 			    cmd_flags.u = TRUE;
 			    break;
+			case 'U':
+			    if(cmd_flags.u == TRUE){
+				error("can't specifiy both -U and -u");
+				usage();
+			    }
+			    cmd_flags.U = TRUE;
+			    break;
 			case 'm':
 			    cmd_flags.m = TRUE;
 			    break;
@@ -412,10 +443,10 @@ char **envp)
 	}
 
 	for(j = 0; j < cmd_flags.nfiles; j++)
-	    ofile_process(files[j], arch_flags, narch_flags, all_archs, FALSE,
+	    ofile_process(files[j], arch_flags, narch_flags, all_archs, TRUE,
 			  cmd_flags.f, TRUE, nm, &cmd_flags);
 	if(cmd_flags.nfiles == 0)
-	    ofile_process("a.out",  arch_flags, narch_flags, all_archs, FALSE,
+	    ofile_process("a.out",  arch_flags, narch_flags, all_archs, TRUE,
 			  cmd_flags.f, TRUE, nm, &cmd_flags);
 
 	if(errors == 0)
@@ -432,7 +463,7 @@ void
 usage(
 void)
 {
-	fprintf(stderr, "Usage: %s [-agnoprumxjlfAP[s segname sectname] [-] "
+	fprintf(stderr, "Usage: %s [-agnopruUmxjlfAP[s segname sectname] [-] "
 		"[-t format] [[-arch <arch_flag>] ...] [file ...]\n", progname);
 	exit(EXIT_FAILURE);
 }
@@ -451,7 +482,7 @@ void *cookie)
     uint32_t ncmds, mh_flags;
     struct cmd_flags *cmd_flags;
     struct process_flags process_flags;
-    unsigned long i, j, k;
+    uint32_t i, j, k;
     struct load_command *lc;
     struct symtab_command *st;
     struct dysymtab_command *dyst;
@@ -462,7 +493,7 @@ void *cookie)
     struct dylib_command *dl;
 
     struct symbol *symbols;
-    unsigned long nsymbols;
+    uint32_t nsymbols;
     struct value_diff *value_diffs;
 
     char *short_name, *has_suffix;
@@ -482,6 +513,13 @@ void *cookie)
 	process_flags.nlibs = 0;
 	process_flags.lib_names = NULL;
 
+	if(ofile->mh == NULL && ofile->mh64 == NULL){
+#ifdef LTO_SUPPORT
+	    if(ofile->lto != NULL)
+		nm_lto(ofile, arch_name, cmd_flags);
+#endif /* LTO_SUPPORT */
+	    return;
+	}
 	st = NULL;
 	dyst = NULL;
 	lc = ofile->load_commands;
@@ -510,13 +548,16 @@ void *cookie)
 	    }
 	    else if((mh_flags & MH_TWOLEVEL) == MH_TWOLEVEL &&
 		    (lc->cmd == LC_LOAD_DYLIB ||
-		     lc->cmd == LC_LOAD_WEAK_DYLIB)){
+		     lc->cmd == LC_LOAD_WEAK_DYLIB ||
+		     lc->cmd == LC_LAZY_LOAD_DYLIB ||
+		     lc->cmd == LC_REEXPORT_DYLIB ||
+		     lc->cmd == LC_LOAD_UPWARD_DYLIB)){
 		process_flags.nlibs++;
 	    }
 	    lc = (struct load_command *)((char *)lc + lc->cmdsize);
 	}
 	if(st == NULL || st->nsyms == 0){
-	    error("no name list");
+	    warning("no name list");
 	    return;
 	}
 	if(process_flags.nsects > 0){
@@ -600,7 +641,11 @@ void *cookie)
 	    j = 0;
 	    lc = ofile->load_commands;
 	    for (i = 0; i < ncmds; i++){
-		if(lc->cmd == LC_LOAD_DYLIB || lc->cmd == LC_LOAD_WEAK_DYLIB){
+		if(lc->cmd == LC_LOAD_DYLIB ||
+		   lc->cmd == LC_LOAD_WEAK_DYLIB ||
+		   lc->cmd == LC_LAZY_LOAD_DYLIB ||
+		   lc->cmd == LC_REEXPORT_DYLIB ||
+		   lc->cmd == LC_LOAD_UPWARD_DYLIB){
 		    dl = (struct dylib_command *)lc;
 		    process_flags.lib_names[j] =
 			(char *)dl + dl->dylib.name.offset;
@@ -622,12 +667,13 @@ void *cookie)
 	/* set names in the symbols to be printed */
 	strings = ofile->object_addr + st->stroff;
 	strsize = st->strsize;
+	compare_lto = FALSE;
 	if(cmd_flags->x == FALSE){
 	    for(i = 0; i < nsymbols; i++){
 		if(symbols[i].nl.n_un.n_strx == 0)
 		    symbols[i].name = "";
-		else if(symbols[i].nl.n_un.n_strx < 0 ||
-			(unsigned long)symbols[i].nl.n_un.n_strx > st->strsize)
+		else if((int)symbols[i].nl.n_un.n_strx < 0 ||
+			(uint32_t)symbols[i].nl.n_un.n_strx > st->strsize)
 		    symbols[i].name = "bad string index";
 		else
 		    symbols[i].name = symbols[i].nl.n_un.n_strx + strings;
@@ -643,7 +689,7 @@ void *cookie)
 		}
 	    }
 	    if(cmd_flags->l == TRUE &&
-	       (long)process_flags.nsect != -1 &&
+	       (int32_t)process_flags.nsect != -1 &&
 	       process_flags.sect_start_symbol == FALSE &&
 	       process_flags.sect_size != 0){
 		symbols = reallocate(symbols,
@@ -657,25 +703,7 @@ void *cookie)
 	}
 
 	/* print header if needed */
-	if((ofile->member_ar_hdr != NULL ||
-	    ofile->dylib_module_name != NULL ||
-	    cmd_flags->nfiles > 1 ||
-	    arch_name != NULL) &&
-	    (cmd_flags->o == FALSE && cmd_flags->A == FALSE)){
-	    if(ofile->dylib_module_name != NULL){
-		printf("\n%s(%s)", ofile->file_name, ofile->dylib_module_name);
-	    }
-	    else if(ofile->member_ar_hdr != NULL){
-		printf("\n%s(%.*s)", ofile->file_name,
-		       (int)ofile->member_name_size, ofile->member_name);
-	    }
-	    else
-		printf("\n%s", ofile->file_name);
-	    if(arch_name != NULL)
-		printf(" (for architecture %s):\n", arch_name);
-	    else
-		printf(":\n");
-	}
+	print_header(ofile, arch_name, cmd_flags);
 
 	/* sort the symbols if needed */
 	if(cmd_flags->p == FALSE && cmd_flags->b == FALSE)
@@ -723,6 +751,136 @@ void *cookie)
 	}
 }
 
+#ifdef LTO_SUPPORT
+/*
+ * In translating the information in an lto bitcode file to something that looks
+ * like what would be in a Mach-O file for use by print_mach_symbols() we use
+ * these sections for the CODE, DATA and RODATA defined symbols.
+ */
+static struct section lto_code_section = { "CODE", "LTO" };
+static struct section lto_data_section = { "DATA", "LTO" };
+static struct section lto_rodata_section = { "RODATA", "LTO" };
+static struct section *lto_sections[3] = {
+	&lto_code_section,
+	&lto_data_section,
+	&lto_rodata_section
+};
+static struct section_64 lto_code_section64 = { "CODE", "LTO" };
+static struct section_64 lto_data_section64 = { "DATA", "LTO" };
+static struct section_64 lto_rodata_section64 = { "RODATA", "LTO" };
+static struct section_64 *lto_sections64[3] = {
+	&lto_code_section64,
+	&lto_data_section64,
+	&lto_rodata_section64
+};
+
+/*
+ * nm_lto() is called by nm() to process an lto bitcode file.
+ */
+static
+void
+nm_lto(
+struct ofile *ofile,
+char *arch_name,
+struct cmd_flags *cmd_flags)
+{
+    uint32_t nsyms, nsymbols, i;
+    struct symbol symbol, *symbols;
+    struct process_flags process_flags;
+
+	process_flags.nsect = -1;
+	if(cmd_flags->segname != NULL &&
+	   strcmp(cmd_flags->segname, "LTO") == 0){
+	    if(strcmp(cmd_flags->sectname, "CODE") == 0)
+		process_flags.nsect = 1;
+	    else if(strcmp(cmd_flags->sectname, "DATA") == 0)
+		process_flags.nsect = 2;
+	    else if(strcmp(cmd_flags->sectname, "RODATA") == 0)
+		process_flags.nsect = 3;
+	}
+	process_flags.sect_addr = 0;
+	process_flags.sect_size = 0;
+	process_flags.sect_start_symbol = FALSE;
+	process_flags.nsects = 3;
+	if((ofile->lto_cputype & CPU_ARCH_ABI64) != CPU_ARCH_ABI64){
+	    process_flags.sections = lto_sections;
+	    process_flags.sections64 = NULL;
+	}
+	else{
+	    process_flags.sections64 = lto_sections64;
+	    process_flags.sections = NULL;
+	}
+	process_flags.text_nsect = 1;
+	process_flags.data_nsect = 2;
+	process_flags.bss_nsect = NO_SECT;
+	process_flags.nlibs = 0;
+	process_flags.lib_names = NULL;
+
+	nsyms = lto_get_nsyms(ofile->lto);
+	symbols = allocate(sizeof(struct symbol) * nsyms);
+
+	nsymbols = 0;
+	for(i = 0; i < nsyms; i++){
+	    symbol.name = lto_symbol_name(ofile->lto, i);
+	    symbol.indr_name = NULL;
+	    lto_get_nlist_64(&(symbol.nl), ofile->lto, i);
+	    if(select_symbol(&symbol, cmd_flags, &process_flags))
+		symbols[nsymbols++] = symbol;
+	}
+
+	print_header(ofile, arch_name, cmd_flags);
+
+	/* reset these as the can be used by compare() with -x */
+	strings = NULL;
+	strsize = 0;
+	compare_lto = TRUE;
+
+	/* sort the symbols if needed */
+	if(cmd_flags->p == FALSE)
+	    qsort(symbols, nsymbols, sizeof(struct symbol),
+		  (int (*)(const void *, const void *))compare);
+
+	/* now print the symbols as specified by the flags */
+	if(cmd_flags->m == TRUE)
+	    print_mach_symbols(ofile, symbols, nsymbols, NULL, 0,
+			       cmd_flags, &process_flags, arch_name);
+	else
+	    print_symbols(ofile, symbols, nsymbols, NULL, 0,
+			  cmd_flags, &process_flags, arch_name, NULL);
+
+	free(symbols);
+}
+#endif /* LTO_SUPPORT */
+
+/* print header if needed */
+static
+void
+print_header(
+struct ofile *ofile,
+char *arch_name,
+struct cmd_flags *cmd_flags)
+{
+	if((ofile->member_ar_hdr != NULL ||
+	    ofile->dylib_module_name != NULL ||
+	    cmd_flags->nfiles > 1 ||
+	    arch_name != NULL) &&
+	    (cmd_flags->o == FALSE && cmd_flags->A == FALSE)){
+	    if(ofile->dylib_module_name != NULL){
+		printf("\n%s(%s)", ofile->file_name, ofile->dylib_module_name);
+	    }
+	    else if(ofile->member_ar_hdr != NULL){
+		printf("\n%s(%.*s)", ofile->file_name,
+		       (int)ofile->member_name_size, ofile->member_name);
+	    }
+	    else
+		printf("\n%s", ofile->file_name);
+	    if(arch_name != NULL)
+		printf(" (for architecture %s):\n", arch_name);
+	    else
+		printf(":\n");
+	}
+}
+
 /*
  * select_symbols returns an allocated array of symbol structs as the symbols
  * that are to be printed based on the flags.  The number of symbols in the
@@ -736,9 +894,9 @@ struct symtab_command *st,
 struct dysymtab_command *dyst,
 struct cmd_flags *cmd_flags,
 struct process_flags *process_flags,
-unsigned long *nsymbols)
+uint32_t *nsymbols)
 {
-    unsigned long i, flags, nest;
+    uint32_t i, flags, nest;
     struct nlist *all_symbols;
     struct nlist_64 *all_symbols64;
     struct symbol *selected_symbols, symbol;
@@ -851,7 +1009,7 @@ unsigned long *nsymbols)
 		    make_symbol_64(&symbol, all_symbols64 + i);
 		if(symbol.nl.n_type == N_BINCL &&
 		   symbol.nl.n_un.n_strx != 0 &&
-		   (unsigned long)symbol.nl.n_un.n_strx < st->strsize &&
+		   (uint32_t)symbol.nl.n_un.n_strx < st->strsize &&
 		   strcmp(cmd_flags->bincl_name,
 			  strings + symbol.nl.n_un.n_strx) == 0){
 		    selected_symbols[(*nsymbols)++] = symbol;
@@ -943,6 +1101,12 @@ struct process_flags *process_flags)
 	    else
 		return(FALSE);
 	}
+	if(cmd_flags->U == TRUE){
+	    if((symbol->nl.n_type == (N_UNDF | N_EXT) &&
+		symbol->nl.n_value == 0) ||
+	       symbol->nl.n_type == (N_PBUD | N_EXT))
+		return(FALSE);
+	}
 	if(cmd_flags->g == TRUE && (symbol->nl.n_type & N_EXT) == 0)
 	    return(FALSE);
 	if(cmd_flags->s == TRUE){
@@ -972,28 +1136,35 @@ void
 print_mach_symbols(
 struct ofile *ofile,
 struct symbol *symbols,
-unsigned long nsymbols,
+uint32_t nsymbols,
 char *strings,
-unsigned long strsize,
+uint32_t strsize,
 struct cmd_flags *cmd_flags,
 struct process_flags *process_flags,
 char *arch_name)
 {
-    unsigned long i, library_ordinal;
-    char *ta_xfmt, *i_xfmt, *spaces;
+    uint32_t i, library_ordinal;
+    char *ta_xfmt, *i_xfmt, *dashes, *spaces;
     uint32_t mh_flags;
 
-	if(ofile->mh != NULL){
+	mh_flags = 0;
+	if(ofile->mh != NULL ||
+	   (ofile->lto != NULL &&
+	    (ofile->lto_cputype & CPU_ARCH_ABI64) != CPU_ARCH_ABI64)){
 	    ta_xfmt = "%08llx";
 	    i_xfmt =  "%08x";
-	    mh_flags = ofile->mh->flags;
+	    if(ofile->mh != NULL)
+		mh_flags = ofile->mh->flags;
 	    spaces = "        ";
+	    dashes = "--------";
 	}
 	else{
 	    ta_xfmt = "%016llx";
 	    i_xfmt =  "%016x";
-	    mh_flags = ofile->mh64->flags;
+	    if(ofile->mh64 != NULL)
+		mh_flags = ofile->mh64->flags;
 	    spaces = "                ";
+	    dashes = "----------------";
 	}
 	for(i = 0; i < nsymbols; i++){
 	    if(cmd_flags->x == TRUE){
@@ -1004,14 +1175,17 @@ char *arch_name)
 		       (unsigned int)(symbols[i].nl.n_desc & 0xffff));
 		if(symbols[i].nl.n_un.n_strx == 0){
 		    printf(i_xfmt, symbols[i].nl.n_un.n_strx);
-		    printf(" (null)");
+		    if(ofile->lto != NULL)
+			printf(" %s", symbols[i].name);
+		    else
+			printf(" (null)");
 		}
-		else if((unsigned long)symbols[i].nl.n_un.n_strx > strsize){
-		    printf(i_xfmt, symbols[i].nl.n_un.n_strx);
+		else if((uint32_t)symbols[i].nl.n_un.n_strx > strsize){
+		    printf("%08x", symbols[i].nl.n_un.n_strx);
 		    printf(" (bad string index)");
 		}
 		else{
-		    printf(i_xfmt, symbols[i].nl.n_un.n_strx);
+		    printf("%08x", symbols[i].nl.n_un.n_strx);
 		    printf(" %s", symbols[i].nl.n_un.n_strx + strings);
 		}
 		if((symbols[i].nl.n_type & N_STAB) == 0 &&
@@ -1080,16 +1254,24 @@ char *arch_name)
 	    if(((symbols[i].nl.n_type & N_TYPE) == N_UNDF &&
 		 symbols[i].nl.n_value == 0) ||
 		 (symbols[i].nl.n_type & N_TYPE) == N_INDR)
-		printf(spaces);
-	    else
-		printf(ta_xfmt, symbols[i].nl.n_value);
+		printf("%s", spaces);
+	    else{
+		if(ofile->lto)
+		    printf("%s", dashes);
+		else
+		    printf(ta_xfmt, symbols[i].nl.n_value);
+	    }
 
 	    switch(symbols[i].nl.n_type & N_TYPE){
 	    case N_UNDF:
 	    case N_PBUD:
 		if((symbols[i].nl.n_type & N_TYPE) == N_UNDF &&
-		   symbols[i].nl.n_value != 0)
+		   symbols[i].nl.n_value != 0){
 		    printf(" (common) ");
+		    if(GET_COMM_ALIGN(symbols[i].nl.n_desc) != 0)
+			printf("(alignment 2^%d) ",
+			       GET_COMM_ALIGN(symbols[i].nl.n_desc));
+		}
 		else{
 		    if((symbols[i].nl.n_type & N_TYPE) == N_PBUD)
 			printf(" (prebound ");
@@ -1118,7 +1300,10 @@ char *arch_name)
 	    case N_SECT:
 		if(symbols[i].nl.n_sect >= 1 &&
 		   symbols[i].nl.n_sect <= process_flags->nsects){
-		    if(ofile->mh != NULL){
+		    if(ofile->mh != NULL ||
+	   	       (ofile->lto != NULL &&
+	    		(ofile->lto_cputype & CPU_ARCH_ABI64) !=
+			 CPU_ARCH_ABI64)){
 			printf(" (%.16s,%.16s) ",
 			       process_flags->sections[
 				    symbols[i].nl.n_sect-1]->segname,
@@ -1152,8 +1337,13 @@ char *arch_name)
 		}
 		else{
 		    if((symbols[i].nl.n_desc & N_WEAK_REF) == N_WEAK_REF ||
-		       (symbols[i].nl.n_desc & N_WEAK_DEF) == N_WEAK_DEF)
-			printf("weak external ");
+		       (symbols[i].nl.n_desc & N_WEAK_DEF) == N_WEAK_DEF){
+			if((symbols[i].nl.n_desc & (N_WEAK_REF | N_WEAK_DEF)) ==
+			   (N_WEAK_REF | N_WEAK_DEF))
+			    printf("weak external automatically hidden ");
+			else
+			    printf("weak external ");
+		    }
 		    else
 			printf("external ");
 		}
@@ -1169,6 +1359,19 @@ char *arch_name)
 	       (symbols[i].nl.n_desc & N_NO_DEAD_STRIP) == N_NO_DEAD_STRIP)
 		    printf("[no dead strip] ");
 
+	    if(ofile->mh_filetype == MH_OBJECT &&
+	       ((symbols[i].nl.n_type & N_TYPE) != N_UNDF) &&
+	       (symbols[i].nl.n_desc & N_SYMBOL_RESOLVER) == N_SYMBOL_RESOLVER)
+		    printf("[symbol resolver] ");
+
+	    if(ofile->mh_filetype == MH_OBJECT &&
+	       ((symbols[i].nl.n_type & N_TYPE) != N_UNDF) &&
+	       (symbols[i].nl.n_desc & N_ALT_ENTRY) == N_ALT_ENTRY)
+		    printf("[alt entry] ");
+
+	    if((symbols[i].nl.n_desc & N_ARM_THUMB_DEF) == N_ARM_THUMB_DEF)
+		    printf("[Thumb] ");
+
 	    if((symbols[i].nl.n_type & N_TYPE) == N_INDR)
 		printf("%s (for %s)", symbols[i].name, symbols[i].indr_name);
 	    else
@@ -1186,7 +1389,7 @@ char *arch_name)
 			    library_ordinal == DYNAMIC_LOOKUP_ORDINAL)
 			printf(" (dynamically looked up)");
 		    else if(library_ordinal-1 >= process_flags->nlibs)
-			printf(" (from bad library ordinal %lu)",
+			printf(" (from bad library ordinal %u)",
 			       library_ordinal);
 		    else
 			printf(" (from %s)", process_flags->lib_names[
@@ -1206,27 +1409,31 @@ void
 print_symbols(
 struct ofile *ofile,
 struct symbol *symbols,
-unsigned long nsymbols,
+uint32_t nsymbols,
 char *strings,
-unsigned long strsize,
+uint32_t strsize,
 struct cmd_flags *cmd_flags,
 struct process_flags *process_flags,
 char *arch_name,
 struct value_diff *value_diffs)
 {
-    unsigned long i;
+    uint32_t i;
     unsigned char c;
-    char *ta_xfmt, *i_xfmt, *spaces, *p;
+    char *ta_xfmt, *i_xfmt, *spaces, *dashes, *p;
 
-	if(ofile->mh != NULL){
+	if(ofile->mh != NULL ||
+	   (ofile->lto != NULL &&
+	    (ofile->lto_cputype & CPU_ARCH_ABI64) != CPU_ARCH_ABI64)){
 	    ta_xfmt = "%08llx";
 	    i_xfmt =  "%08x";
 	    spaces = "        ";
+	    dashes = "--------";
 	}
 	else{
 	    ta_xfmt = "%016llx";
 	    i_xfmt =  "%016x";
 	    spaces = "                ";
+	    dashes = "----------------";
 	}
 
 	for(i = 0; i < nsymbols; i++){
@@ -1238,14 +1445,17 @@ struct value_diff *value_diffs)
 		       (unsigned int)(symbols[i].nl.n_desc & 0xffff));
 		if(symbols[i].nl.n_un.n_strx == 0){
 		    printf(i_xfmt, symbols[i].nl.n_un.n_strx);
-		    printf(" (null)");
+		    if(ofile->lto != NULL)
+			printf(" %s", symbols[i].name);
+		    else
+			printf(" (null)");
 		}
-		else if((unsigned long)symbols[i].nl.n_un.n_strx > strsize){
-		    printf(i_xfmt, symbols[i].nl.n_un.n_strx);
+		else if((uint32_t)symbols[i].nl.n_un.n_strx > strsize){
+		    printf("%08x", symbols[i].nl.n_un.n_strx);
 		    printf(" (bad string index)");
 		}
 		else{
-		    printf(i_xfmt, symbols[i].nl.n_un.n_strx);
+		    printf("%08x", symbols[i].nl.n_un.n_strx);
 		    printf(" %s", symbols[i].nl.n_un.n_strx + strings);
 		}
 		if((symbols[i].nl.n_type & N_STAB) == 0 &&
@@ -1429,13 +1639,16 @@ struct value_diff *value_diffs)
 		c = toupper(c);
 	    if(cmd_flags->u == FALSE && cmd_flags->j == FALSE){
 		if(c == 'u' || c == 'U' || c == 'i' || c == 'I')
-		    printf(spaces);
+		    printf("%s", spaces);
 		else{
 		    if(cmd_flags->v && value_diffs != NULL){
 			printf(ta_xfmt, value_diffs[i].size);
 			printf(" ");
 		    }
-		    printf(ta_xfmt, symbols[i].nl.n_value);
+		    if(ofile->lto)
+			printf("%s", dashes);
+		    else
+			printf(ta_xfmt, symbols[i].nl.n_value);
 		}
 		printf(" %c ", c);
 	    }
@@ -1459,6 +1672,7 @@ static const struct stabnames stabnames[] = {
     { N_STSYM, "STSYM" },
     { N_LCSYM, "LCSYM" },
     { N_BNSYM, "BNSYM" },
+    { N_AST,   "AST" },
     { N_OPT,   "OPT" },
     { N_RSYM,  "RSYM" },
     { N_SLINE, "SLINE" },
@@ -1528,12 +1742,12 @@ struct symbol *p2)
 	     */
 	}
 
-	if(cmd_flags.x == TRUE){
-	    if((unsigned long)p1->nl.n_un.n_strx > strsize ||
-	       (unsigned long)p2->nl.n_un.n_strx > strsize){
-		if((unsigned long)p1->nl.n_un.n_strx > strsize)
+	if(cmd_flags.x == TRUE && compare_lto == FALSE){
+	    if((uint32_t)p1->nl.n_un.n_strx > strsize ||
+	       (uint32_t)p2->nl.n_un.n_strx > strsize){
+		if((uint32_t)p1->nl.n_un.n_strx > strsize)
 		    r = -1;
-		else if((unsigned long)p2->nl.n_un.n_strx > strsize)
+		else if((uint32_t)p2->nl.n_un.n_strx > strsize)
 		    r = 1;
 	    }
 	    else
